@@ -1,13 +1,13 @@
 # -*- coding: utf-8 -*-
 """Field classes for various types of data.
 """
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import datetime as dt
 import uuid
 import warnings
-from decimal import Decimal
-from functools import partial
+import decimal
+from operator import attrgetter
 
 from marshmallow import validate, utils, class_registry
 from marshmallow.base import FieldABC, SchemaABC
@@ -32,6 +32,7 @@ __all__ = [
     'UUID',
     'Number',
     'Integer',
+    'Decimal',
     'Boolean',
     'FormattedString',
     'Float',
@@ -49,6 +50,8 @@ __all__ = [
     'Method',
     'Function',
     'Select',
+    'QuerySelect',
+    'QuerySelectList',
     'Enum',
     'Str',
     'Bool',
@@ -215,8 +218,8 @@ class Unmarshaller(object):
         for validator_func in validators:
             try:
                 if validator_func(output) is False:
-                    func_name = utils.get_func_name(validator_func)
-                    raise ValidationError(u'Schema validator {0}({1}) is False'.format(
+                    func_name = utils.get_callable_name(validator_func)
+                    raise ValidationError('Schema validator {0}({1}) is False'.format(
                         func_name, dict(output)
                     ))
             except ValidationError as err:
@@ -229,7 +232,12 @@ class Unmarshaller(object):
                     field_obj = None
                 if strict:
                     raise UnmarshallingError(err, field=field_obj, field_name=field_name)
-                self.errors.setdefault(field_name, []).append(text_type(err))
+                if isinstance(err.messages, (list, tuple)):
+                    self.errors.setdefault(field_name, []).extend(err.messages)
+                elif isinstance(err.messages, dict):
+                    self.errors.setdefault(field_name, []).append(err.messages)
+                else:
+                    self.errors.setdefault(field_name, []).append(text_type(err))
         return output
 
     def deserialize(self, data, fields_dict, many=False, validators=None,
@@ -325,7 +333,7 @@ class Field(FieldABC):
     #  for those fields
     _CHECK_ATTRIBUTE = True
     _creation_index = 0
-    # Values that are skipped by `Marshaller` if ``skip_missing=True``
+    #: Values that are skipped by `Marshaller` if ``skip_missing=True``
     SKIPPABLE_VALUES = (None, )
 
     def __init__(self, default=None, attribute=None, error=None,
@@ -377,15 +385,18 @@ class Field(FieldABC):
         """
         errors = []
         for validator in self.validators:
-            func_name = utils.get_func_name(validator)
-            msg = u'Validator {0}({1}) is False'.format(
+            func_name = utils.get_callable_name(validator)
+            msg = 'Validator {0}({1}) is False'.format(
                 func_name, value
             )
             try:
                 if validator(value) is False:
                     raise ValidationError(getattr(self, 'error', None) or msg)
             except ValidationError as err:
-                errors.extend(err.messages)
+                if isinstance(err.messages, dict):
+                    errors.append(err.messages)
+                else:
+                    errors.extend(err.messages)
         if errors:
             raise ValidationError(errors)
 
@@ -393,7 +404,7 @@ class Field(FieldABC):
         """Utility method to invoke a function and raise ``exception_class`` if an error
         occurs.
 
-        :param callable function: Function to call. Must take no arguments.
+        :param callable func: Function to call. Must take no arguments.
         :param Exception exception_class: Type of exception to raise when an error occurs.
         """
         try:
@@ -664,7 +675,12 @@ class UUID(String):
     """A UUID field."""
 
     def _deserialize(self, value):
-        return uuid.UUID(value)
+        msg = 'Could not deserialize {0!r} to a UUID object.'.format(value)
+        err = UnmarshallingError(getattr(self, 'error', None) or msg)
+        try:
+            return uuid.UUID(value)
+        except (ValueError, AttributeError):
+            raise err
 
 
 class Number(Field):
@@ -682,23 +698,27 @@ class Number(Field):
             error=error, **kwargs)
 
     def _format_num(self, value):
-        """Return the correct value for a number, given the passed in
-        arguments to __init__.
-        """
-        if self.as_string:
-            return repr(self.num_type(value))
-        else:
-            return self.num_type(value)
+        """Return the number value for value, given this field's `num_type`."""
+        return self.num_type(value)
 
     def _validated(self, value, exception_class):
         """Format the value or raise ``exception_class`` if an error occurs."""
+        if value is None:
+            return self.default
         try:
-            if value is None:
-                return self.default
             return self._format_num(value)
-        except (TypeError, ValueError) as err:
+        except (TypeError, ValueError, decimal.InvalidOperation) as err:
             raise exception_class(getattr(self, 'error', None) or err)
 
+    def serialize(self, attr, obj, accessor=None):
+        """Pulls the value for the given key from the object and returns the
+        serialized number representation. Return a string if `self.as_string=True`,
+        othewise return this field's `num_type`. Receives the same `args` and `kwargs`
+        as `Field`.
+        """
+        ret = Field.serialize(self, attr, obj, accessor=accessor)
+        return str(ret) if self.as_string else ret
+
     def _serialize(self, value, attr, obj):
         return self._validated(value, MarshallingError)
 
@@ -720,6 +740,49 @@ class Integer(Number):
         super(Integer, self).__init__(default=default, attribute=attribute,
             error=error, **kwargs)
 
+
+class Decimal(Number):
+    """A field that (de)serializes to the Python ``decimal.Decimal`` type.
+    It's safe to use when dealing with money values, percentages, ratios
+    or other numbers where precision is critical.
+
+    .. warning::
+
+        This field serializes to a `decimal.Decimal` object by default. If you need
+        to render your data as JSON, keep in mind that the `json` module from the
+        standard library does not encode `decimal.Decimal`. Therefore, you must use
+        a JSON library that can handle decimals, such as `simplejson`, or serialize
+        to a string by passing ``as_string=True``.
+
+    :param int places: How many decimal places to quantize the value. If `None`, does
+        not quantize the value.
+    :param rounding: How to round the value during quantize, for example
+        `decimal.ROUND_UP`. If None, uses the rounding value from
+        the current thread's context.
+    :param default: The value this field defaults to. If not specified is the
+        `decimal.Decimal` zero.
+    :param bool as_string: If True, serialize to a string instead of a Python
+        `decimal.Decimal` type.
+    :param kwargs: The same keyword arguments that :class:`Number` receives.
+
+    .. versionadded:: 1.2.0
+    """
+
+    num_type = decimal.Decimal
+
+    def __init__(self, places=None, rounding=None, default=decimal.Decimal(),
+                 as_string=False, **kwargs):
+        self.places = decimal.Decimal((0, (1,), -places)) if places is not None else None
+        self.rounding = rounding
+        super(Decimal, self).__init__(default=default, as_string=as_string, **kwargs)
+
+    def _format_num(self, value):
+        num = decimal.Decimal(value)
+        if self.places is not None:
+            num = num.quantize(self.places, rounding=self.rounding)
+        return num
+
+
 class Boolean(Field):
     """A boolean field.
 
@@ -800,9 +863,16 @@ class Arbitrary(Number):
     ex: 634271127864378216478362784632784678324.23432
 
     :param kwargs: The same keyword arguments that :class:`Number` receives.
+
+    .. deprecated:: 1.2.0
+        Use `Decimal` instead.
     """
     # No as_string param
     def __init__(self, default='0', attribute=None, **kwargs):
+        warnings.warn(
+            'The Arbitrary field is deprecated. Use the Decimal field instead.',
+            category=DeprecationWarning
+        )
         super(Arbitrary, self).__init__(default=default, attribute=attribute, **kwargs)
 
     def _validated(self, value, exception_class):
@@ -839,7 +909,8 @@ DATEFORMAT_DESERIALIZATION_FUNCS = {
 
 class DateTime(Field):
     """A formatted datetime string in UTC.
-        ex. ``"Sun, 10 Nov 2013 07:23:45 -0000"``
+
+    Example: ``'2014-12-22T03:12:58.019077+00:00'``
 
     :param str format: Either ``"rfc"`` (for RFC822), ``"iso"`` (for ISO8601),
         or a date format string. If `None`, defaults to "iso".
@@ -873,19 +944,17 @@ class DateTime(Field):
                 return value.strftime(self.dateformat)
 
     def _deserialize(self, value):
-        err = UnmarshallingError(
-            'Cannot deserialize {0!r} to a datetime'.format(value)
-        )
+        msg = 'Could not deserialize {0!r} to a datetime object.'.format(value)
+        err = UnmarshallingError(getattr(self, 'error', None) or msg)
+        if not value:  # Falsy values, e.g. '', None, [] are not valid
+            raise err
         self.dateformat = self.dateformat or self.DEFAULT_FORMAT
         func = DATEFORMAT_DESERIALIZATION_FUNCS.get(self.dateformat)
         if func:
             try:
                 return func(value)
-            except TypeError:
+            except (TypeError, AttributeError, ValueError):
                 raise err
-            except (AttributeError, ValueError) as err:
-                msg = 'Could not deserialize {0!r} to a datetime object.'.format(value)
-                raise UnmarshallingError(getattr(self, 'error', None) or msg)
         elif utils.dateutil_available:
             try:
                 return utils.from_datestring(value)
@@ -925,11 +994,14 @@ class Time(Field):
 
     def _deserialize(self, value):
         """Deserialize an ISO8601-formatted time to a :class:`datetime.time` object."""
+        msg = 'Could not deserialize {0!r} to a time object.'.format(value)
+        err = UnmarshallingError(getattr(self, 'error', None) or msg)
+        if not value:   # falsy values are invalid
+            raise err
         try:
             return utils.from_iso_time(value)
-        except (TypeError, ValueError):
-            msg = 'Could not deserialize {0!r} to a time object.'.format(value)
-            raise UnmarshallingError(getattr(self, 'error', None) or msg)
+        except (AttributeError, TypeError, ValueError):
+            raise err
 
 class Date(Field):
     """ISO8601-formatted date string.
@@ -949,11 +1021,14 @@ class Date(Field):
         """Deserialize an ISO8601-formatted date string to a
         :class:`datetime.date` object.
         """
+        msg = 'Could not deserialize {0!r} to a date object.'.format(value)
+        err = UnmarshallingError(getattr(self, 'error', None) or msg)
+        if not value:  # falsy values are invalid
+            raise err
         try:
             return utils.from_iso_date(value)
-        except (TypeError, ValueError):
-            msg = 'Could not deserialize {0!r} to a date object.'.format(value)
-            raise UnmarshallingError(getattr(self, 'error', None) or msg)
+        except (AttributeError, TypeError, ValueError):
+            raise err
 
 
 class TimeDelta(Field):
@@ -977,7 +1052,7 @@ class TimeDelta(Field):
         """
         try:
             return dt.timedelta(seconds=float(value))
-        except (AttributeError, ValueError):
+        except (TypeError, AttributeError, ValueError):
             msg = 'Could not deserialize {0!r} to a timedelta object.'.format(value)
             raise UnmarshallingError(getattr(self, 'error', None) or msg)
 
@@ -986,13 +1061,20 @@ class Fixed(Number):
     """A fixed-precision number as a string.
 
     :param kwargs: The same keyword arguments that :class:`Number` receives.
+
+    .. deprecated:: 1.2.0
+        Use `Decimal` instead.
     """
 
     def __init__(self, decimals=5, default='0.000', attribute=None, error=None,
                  *args, **kwargs):
+        warnings.warn(
+            'The Fixed field is deprecated. Use the Decimal field instead.',
+            category=DeprecationWarning
+        )
         super(Fixed, self).__init__(default=default, attribute=attribute, error=error,
                             *args, **kwargs)
-        self.precision = Decimal('0.' + '0' * (decimals - 1) + '1')
+        self.precision = decimal.Decimal('0.' + '0' * (decimals - 1) + '1')
 
     def _serialize(self, value, attr, obj):
         return self._validated(value, MarshallingError)
@@ -1018,12 +1100,20 @@ class Price(Fixed):
     """A Price field with fixed precision.
 
     :param kwargs: The same keyword arguments that :class:`Fixed` receives.
+
+    .. deprecated:: 1.2.0
+        Use `Decimal` instead.
     """
     def __init__(self, decimals=2, default='0.00', **kwargs):
+        warnings.warn(
+            'The Price field is deprecated. Use the Decimal field for dealing with '
+            'money values.',
+            category=DeprecationWarning
+        )
         super(Price, self).__init__(decimals=decimals, default=default, **kwargs)
 
 class ValidatedField(Field):
-    """A field that is validates input on serialization."""
+    """A field that validates input on serialization."""
 
     def _validated(self, value):
         raise NotImplementedError('Must implement _validate method')
@@ -1048,11 +1138,11 @@ class Url(ValidatedField):
         self.relative = relative
         # Insert validation into self.validators so that multiple errors can be
         # stored.
-        self.validators.insert(0, partial(validate.url,
-            relative=self.relative, error=getattr(self, 'error')))
+        self.validators.insert(0, validate.URL(relative=self.relative,
+            error=getattr(self, 'error')))
 
     def _validated(self, value):
-        return validate.url(value, relative=self.relative, error=getattr(self, 'error'))
+        return validate.URL(relative=self.relative, error=getattr(self, 'error'))(value)
 
 URL = Url
 
@@ -1065,10 +1155,10 @@ class Email(ValidatedField):
         super(Email, self).__init__(*args, **kwargs)
         # Insert validation into self.validators so that multiple errors can be
         # stored.
-        self.validators.insert(0, partial(validate.email, error=getattr(self, 'error')))
+        self.validators.insert(0, validate.Email(error=getattr(self, 'error')))
 
     def _validated(self, value):
-        return validate.email(value, error=getattr(self, 'error'))
+        return validate.Email(error=getattr(self, 'error'))(value)
 
 
 class Method(Field):
@@ -1189,6 +1279,130 @@ class Select(Field):
     def _deserialize(self, value):
         return self._validated(value, UnmarshallingError)
 
+
+class QuerySelect(Field):
+    """A field that (de)serializes an ORM-mapped object to its primary
+    (or otherwise unique) key and vice versa. A nonexistent key will
+    result in a validation error. This field is ORM-agnostic.
+
+    Example: ::
+
+        query = session.query(User).order_by(User.id).all
+        keygetter = 'id'
+        field = fields.QuerySelect(query, keygetter)
+
+    :param callable query: The query which will be executed at each
+        (de)serialization to find the list of valid objects and keys.
+    :param keygetter: Can be a callable or a string. In the former case, it must
+        be a one-argument callable which returns a unique comparable
+        key. In the latter case, the string specifies the name of
+        an attribute of the ORM-mapped object.
+    :param str error: Error message stored upon validation failure.
+    :param kwargs: The same keyword arguments that :class:`Field` receives.
+
+    .. versionadded:: 1.2.0
+    """
+    def __init__(self, query, keygetter, error=None, **kwargs):
+        self.query = query
+        self.keygetter = keygetter if callable(keygetter) else attrgetter(keygetter)
+        super(QuerySelect, self).__init__(error=error, **kwargs)
+
+    def keys(self):
+        """Return a generator over the valid keys."""
+        return (self.keygetter(item) for item in self.query())
+
+    def results(self):
+        """Return a generator over the query results."""
+        return (item for item in self.query())
+
+    def pairs(self):
+        """Return a generator over the (key, result) pairs."""
+        return ((self.keygetter(item), item) for item in self.query())
+
+    def labels(self, labelgetter=text_type):
+        """Return a generator over the (key, label) pairs, where
+        label is a string associated with each query result. This
+        convenience method is useful to populate, for instance,
+        a form select field.
+
+        :param labelgetter: Can be a callable or a string. In the former case,
+            it must be a one-argument callable which returns the label text. In the
+            latter case, the string specifies the name of an attribute of
+            the ORM-mapped object. If not provided the ORM-mapped object's
+            `__str__` or `__unicode__` method will be used.
+        """
+        labelgetter = labelgetter if callable(labelgetter) else attrgetter(labelgetter)
+        return ((self.keygetter(item), labelgetter(item)) for item in self.query())
+
+    def _serialize(self, value, attr, obj):
+        value = self.keygetter(value)
+
+        for key in self.keys():
+            if key == value:
+                return value
+
+        error = getattr(self, 'error', None) or 'Invalid object.'
+        raise MarshallingError(error)
+
+    def _deserialize(self, value):
+        for key, result in self.pairs():
+            if key == value:
+                return result
+
+        error = getattr(self, 'error', None) or 'Invalid key.'
+        raise UnmarshallingError(error)
+
+
+class QuerySelectList(QuerySelect):
+    """A field that (de)serializes a list of ORM-mapped objects to
+    a list of their primary (or otherwise unique) keys and vice
+    versa. If any of the items in the list cannot be found in the
+    query, this will result in a validation error. This field
+    is ORM-agnostic.
+
+    :param callable query: Same as :class:`QuerySelect`.
+    :param keygetter: Same as :class:`QuerySelect`.
+    :param str error: Error message stored upon validation failure.
+    :param kwargs: The same keyword arguments that :class:`Field` receives.
+
+    .. versionadded:: 1.2.0
+    """
+    def _serialize(self, value, attr, obj):
+        items = [self.keygetter(v) for v in value]
+
+        if not items:
+            return []
+
+        keys = list(self.keys())
+
+        for item in items:
+            try:
+                keys.remove(item)
+            except ValueError:
+                error = getattr(self, 'error', None) or 'Invalid objects.'
+                raise MarshallingError(error)
+
+        return items
+
+    def _deserialize(self, value):
+        if not value:
+            return []
+
+        keys, results = (list(t) for t in zip(*self.pairs()))
+        items = []
+
+        for val in value:
+            try:
+                index = keys.index(val)
+            except ValueError:
+                error = getattr(self, 'error', None) or 'Invalid keys.'
+                raise UnmarshallingError(error)
+            else:
+                del keys[index]
+                items.append(results.pop(index))
+
+        return items
+
 # Aliases
 Enum = Select
 Str = String
