@@ -1,9 +1,10 @@
 # -*- coding: utf-8 -*-
 """The :class:`Schema` class, including its metaclass and options (class Meta)."""
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import copy
 import datetime as dt
+import decimal
 import json
 import types
 import uuid
@@ -17,9 +18,9 @@ from marshmallow.compat import (with_metaclass, iteritems, text_type,
 from marshmallow.orderedset import OrderedSet
 
 
-#: Return type of :meth:`Schema.dump`
+#: Return type of :meth:`Schema.dump` including serialized data and errors
 MarshalResult = namedtuple('MarshalResult', ['data', 'errors'])
-#: Return type of :meth:`Schema.load`
+#: Return type of :meth:`Schema.load`, including deserialized data and errors
 UnmarshalResult = namedtuple('UnmarshalResult', ['data', 'errors'])
 
 def _get_fields(attrs, field_class, pop=False, ordered=False):
@@ -71,6 +72,7 @@ class SchemaMeta(type):
     a ``_declared_fields`` attribute, which is a dictionary mapping attribute
     names to field objects.
     """
+    FUNC_LISTS = ('__validators__', '__data_handlers__', '__preprocessors__')
 
     def __new__(mcs, name, bases, attrs):
         meta = attrs.get('Meta')
@@ -81,8 +83,14 @@ class SchemaMeta(type):
         dict_cls = OrderedDict if ordered else dict
         klass._declared_fields = dict_cls(fields)
         class_registry.register(name, klass)
+        # Need to copy validators, data handlers, and preprocessors lists
+        # so they are not shared among subclasses and ancestors
+        for attr in mcs.FUNC_LISTS:
+            attr_copy = copy.copy(getattr(klass, attr))
+            setattr(klass, attr, attr_copy)
         return klass
 
+
 class SchemaOpts(object):
     """class Meta options for the :class:`Schema`. Defines defaults."""
 
@@ -168,6 +176,7 @@ class BaseSchema(base.SchemaABC):
         dt.time: fields.Time,
         dt.date: fields.Date,
         dt.timedelta: fields.TimeDelta,
+        decimal.Decimal: fields.Decimal,
     }
 
     OPTIONS_CLASS = SchemaOpts
@@ -483,7 +492,7 @@ class BaseSchema(base.SchemaABC):
             self.__error_handler__(self._unmarshal.errors, data)
         return UnmarshalResult(data=result, errors=errors)
 
-    def loads(self, json_data, many=False, *args, **kwargs):
+    def loads(self, json_data, many=None, *args, **kwargs):
         """Same as :meth:`load`, except it takes a JSON string as input.
 
         :param str json_data: A JSON string of the data to deserialize.
